/*
 * Copyright (C) 1997-2004 Andre Noll <noll@mathematik.tu-darmstadt.de>
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */


#include "server.h"

static FILE *mp3_stream = NULL;
static pid_t stream_writer_pid;
extern struct mp3_meta_data *mmd;
extern struct gengetopt_args_info conf;
extern char *afs_buf;

/*
 * compute human readable string containing
 * afs_status for given integer value
 */
void afs_status_tohuman(int flags, char *msg) 
{
	char *txt;

	if (flags & AFS_PLAYING)
		txt = "playing";
	else if (flags & AFS_NEXT)
		txt = "stopped";
	else 
		txt = "paused";
	sprintf(msg, "%s (%c%c%c%c)", txt,
		(flags & AFS_PLAYING)?	'P' : '_',
		(flags & AFS_NOMORE)?	'O' : '_',
		(flags & AFS_NEXT)?	'N' : '_',
		(flags & AFS_REPOS)?	'R' : '_'
	);
}

/*
 * Read mp3 information from mmd->filename to mmd->mp3
 */
static int mp3_info(void)
{
	mp3info *mp3 = &(mmd->mp3);
	int ret = -1;

	mp3->vbr = 0; /* seems to be nec. */
	if (mp3_stream) {
		mp3->filename = mmd->filename;
		mp3->file = mp3_stream;
		mp3->id3.title[0] = '\0';
		mp3->id3.artist[0] = '\0';
		mp3->id3.album[0] = '\0';
		mp3->id3.comment[0] = '\0';
		mp3->id3.year[0] = '\0';
		
		ret = get_mp3_info(mp3);
		rewind(mp3->file);
	}
	return ret;
}

int update_dbinfo(void)
{
	FILE *pipe;
	char string[STRINGSIZE];
	char *info_cmd[] = {"info", mmd->filename, NULL};
	int n, i;

	mmd->afs_status_flags &= ~AFS_REREAD_DB_INFO;
	clog(DEBUG, "%s: getting dbtool info %s\n", __func__, mmd->filename);
	if (!(pipe = popen_read_dbtool(info_cmd))) {
		clog(ERROR, "%s: popen_read_dbtool failed\n", __func__); 
		return -1;
	}

	n = 0; /* number of bytes already read from pipe */
	i = 0; /* number of dbinfo lines */
	mmd->dbinfo[0] = '\0';
	while (fgets(string, MAXLINE, pipe)) {
		char prefix[MAXLINE];

		i++;
		sprintf(prefix, "dbinfo%d:", i);
		strncat(mmd->dbinfo, prefix, 
			MMD_DBINFO_SIZE - strlen(mmd->dbinfo));
		strncat(mmd->dbinfo, string, 
			MMD_DBINFO_SIZE - strlen(mmd->dbinfo));
		n += strlen(string); 
		clog(INFO, "%s: %s", __func__, string);
	}
	pclose(pipe);
	clog(DEBUG, "%s: Read %i byte of info and put %i byte "
		"into mmd->dbinfo\n", __func__, n, strlen(mmd->dbinfo));
	return 1;
}

/* 
 * upddate shared mem 
 */
static int update_mmd(struct stat *file_status)
{
	FILE *pipe;
	char *us_cmd[] = {"us", mmd->filename, NULL};

	clog(DEBUG, "%s: initialising mmd struct \n", __func__);
//	strncpy(mmd->filename, filename, _POSIX_PATH_MAX);
	mmd->size = file_status->st_size; 
	strncpy(mmd->mtime, ctime(&file_status->st_mtime), MAXLINE);
	clog(NOTICE, "%s: Next song: %s\n", __func__, mmd->filename);
	clog(INFO, "%s: size: %lu, mtime: %s", __func__, mmd->size, mmd->mtime); 
	if (mp3_info() < 0) {
		clog(ERROR, "%s: get_mp3_info failed\n", __func__);
		return -1;
	}

	clog(INFO, "%s: current stream: %s, num_played: %u\n", __func__,
		mmd->stream_list[mmd->stream], mmd->num_played);
	clog(INFO, "%s: length: %i:%02i, %i KBit/s, %i KHz (%s)\n",
		__func__,
		mmd->mp3.seconds / 60,
		mmd->mp3.seconds % 60,
		header_bitrate(&(mmd->mp3.header)),
		header_frequency(&(mmd->mp3.header)),
		header_mode(&(mmd->mp3.header))
	);
	mmd->total = 0;
	clog(DEBUG, "%s: mmd.length=%lu, mmd.size=%lu\n", __func__,
		mmd->mp3.seconds, mmd->size);

	if (update_dbinfo() < 0)
		return -1;

	clog(DEBUG, "%s: calling dbtool to update song\n", __func__);
	if (!(pipe = popen_read_dbtool(us_cmd))) {
		clog(ERROR, "%s: popen_read_dbtool failed\n", __func__); 
		return -1;
	}
	pclose(pipe);
	mmd->old_stream = mmd->stream;
	return 1;
}

/* 
 * get next song via external command dbtool fut. Returns -1 if no valid song
 * could be found, Otherwise, mp3_stream contains valid file and 1 is 
 * returned.
 */
int get_song(void)
{
	FILE *pipe;
	char buf[MAXLINE];
	int ret = -1;
	char *strptr;
	struct stat file_status;
	char *fut_cmd[] = {"fut", "100", mmd->stream_list[mmd->stream], NULL};

	clog(DEBUG, "%s: Getting next song\n", __func__);
	if (!(pipe = popen_read_dbtool(fut_cmd))) {
		clog(ERROR, "%s: popen_read_dbtool failed\n", __func__); 
		goto out;
	}
	while ((fgets(buf, MAXLINE - 1, pipe))) {
		char *name;
		char tmp[MAXLINE] = "";
		int score, pos = 0;


		/* cut trailing \n */
		if ((strptr = strchr(buf, '\n'))) 
			*strptr = '\0';
		sscanf(buf, "%d%n%s", &score, &pos, tmp);
		if (!pos || !tmp[0])
			continue;
		name = buf + pos + 1;
		clog(DEBUG, "%s: Opening %s\n", __func__, name);
		if (!(mp3_stream = fopen(name, "r"))) {
			clog(DEBUG, "%s: Can't open %s\n", __func__, name);
			continue;
		}
		if (fstat(fileno(mp3_stream), &file_status) == -1) { 
			clog(ERROR, "%s: Error, cannot stat %s\n", 
				__func__, name);
			fclose(mp3_stream);
			continue; 
		}
		strcpy(mmd->filename, name);
		mmd->score = score;
		if (update_mmd(&file_status) < 0)
			continue; 
		mmd->num_played++;
		ret = 0;
		break;
	}
out:	
	if (pipe)
		pclose(pipe);
	if (ret < 0) {
		clog(ERROR, "%s: dbtool fut didn't find any valid file\n",
			__func__);
	}
	return ret;
}

static FILE *stream_writer_pipe;

FILE *get_stream_writer_pipe(char *cmd)
{
	if (!stream_writer_pipe)
		stream_writer_pipe = popen_write_cmdline_pid(&stream_writer_pid, cmd);
	if (!stream_writer_pipe) {
		clog(EMERG, "%s: can't open stream_writer_pipe\n", __func__);
		exit(EXIT_FAILURE);
	}
	return stream_writer_pipe;
}

static void kill_stream_writer(void)
{
	clog(DEBUG, "%s: killing stream writer\n", __func__);
	if (stream_writer_pipe) {
		pclose(stream_writer_pipe);
		stream_writer_pipe = NULL;
	}
	if (stream_writer_pid) {
		kill(stream_writer_pid, SIGTERM);
		stream_writer_pid = 0;
	}
}

void do_get_song(void)
{
	if (get_song() < 0)
		mmd->afs_status_flags = AFS_NEXT;
}

/* 
 * called from main when playing, just before main calls select 
 */
void afs_preselect(void)
{
	int flags = mmd->afs_status_flags;

	mmd->afs_status_flags &= (~(AFS_NEXT|AFS_REPOS));
	
	if (!mp3_stream)
		return do_get_song();
	if (flags & AFS_NEXT) {
		kill_stream_writer();
		fclose(mp3_stream);
		mp3_stream = NULL;
		do_get_song();
		return;
	}
	if ((flags & AFS_REPOS)) {
		kill_stream_writer();
		clog(DEBUG, "%s: jmp to %i\n", __func__, mmd->total);
		fseek(mp3_stream, mmd->total, SEEK_SET);
	}
}
/*
 * Status is playing and stream_writer_pipe is ready for writing. So we read 
 * one chunk of data and send it to pipe.
 */
void afs_send_chunk(int fd)
{
	ssize_t soc, ret;
	
	if ((soc = fread(afs_buf, 1, conf.afs_buf_arg, mp3_stream)) <= 0) {
		if (soc < 0)
			clog(ERROR, "%s: read error (%s)\n", __func__, 
				strerror(errno));
		else
			clog(DEBUG, "%s: Song finished (flags:%x)\n", __func__,
				mmd->afs_status_flags);
		fclose(mp3_stream);
		mp3_stream = NULL;
		if ((mmd->afs_status_flags & AFS_NOMORE))
			mmd->afs_status_flags = AFS_NEXT;
		else
			do_get_song();
		return;
	}
repeat:
	clog(PETER_B, "%s: sending %d bytes to fd %d\n", __func__, soc, fd);
	ret = write(fd, afs_buf, soc);
	if (ret <= 0) {
		kill_stream_writer();
		if (ret < 0)
			clog(ERROR, "%s: write error (%s), soc=%i\n", 
				__func__, strerror(errno), soc);
		return;
	}
	if (ret < soc) {
		clog(WARNING, "%s: short write (%i/%i, afs_buf=%i), rewriting\n", 
			__func__, ret, soc, conf.afs_buf_arg);
		memmove(afs_buf, afs_buf + ret, soc - ret);
		soc -= ret;
		goto repeat;
	}
	mmd->total += soc; 
	mmd->afs_status_flags |= AFS_PLAYING;
}
