/*
 * Copyright (C) 2004 Andre Noll <noll@mathematik.tu-darmstadt.de>
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "para.h"
#include "config.h"
#include "client.cmdline.h"

struct gengetopt_args_info args_info;

/*
 * log function
 */
void clog(int ll, char* fmt,...)
{
	va_list argp;
	FILE *outfd;

	/* ignore log message if loglevel is not high enough */
	if (ll < args_info.loglevel_arg)
		return;
	if (ll < WARNING)
		outfd = stdout;
	else
		outfd = stderr;
	va_start(argp, fmt);
	vfprintf(stdout, fmt, argp);
	va_end(argp);
}

void get_options(int argc, char *argv[], char **config_file)
{
	char *home = getenv("HOME");
	static char default_config_file[_POSIX_PATH_MAX] = "";
	struct stat statbuf;
	int ret;

	cmdline_parser(argc, argv, &args_info);
	if (args_info.config_file_given) {
		sprintf(default_config_file, "%s/.paraslash/client.conf",
			home);
		*config_file = default_config_file;
	} else
		*config_file = args_info.config_file_arg;
	ret = stat(*config_file, &statbuf);
	if (ret && args_info.config_file_given) {
		fprintf(stderr, "Can't stat config file %s\n",
			args_info.config_file_arg);
		exit(EXIT_FAILURE);
	}
	if (!ret)
		cmdline_parser_configfile(*config_file, &args_info, 0, 0);
}

/*
 * MAIN
 */
int main(int argc, char *argv[]) {

	int sockfd;
	ssize_t numbytes;
	struct hostent *he;
	struct sockaddr_in their_addr;
	char buf[STRINGSIZE];
	char header[STRINGSIZE];
	char *config_file;

	get_options(argc, argv, &config_file);
	if (args_info.loglevel_arg <= NOTICE)
		cmdline_parser_print_version();
	clog(INFO, "%s:"
		"Current loglevel: %d\n"
		"Using config_file: %s\n"
		"Connecting to %s:%d\n",
		__func__,
		args_info.loglevel_arg,
		config_file,
		args_info.hostname_arg,
		args_info.server_port_arg
	);
	
	/* get the host info */
	clog(NOTICE, "%s: Getting host info of %s\n", __func__,
		args_info.hostname_arg);
	if (!(he = get_host_info(args_info.hostname_arg)))
		exit(EXIT_FAILURE);
	/* get new socket */
	if ((sockfd = get_socket()) < 0)
		exit(EXIT_FAILURE);
	/* init their_addr */
	init_sockaddr(&their_addr, args_info.server_port_arg, he);
	/* Connect */
	clog(NOTICE, "%s: Connecting to %s...\n", __func__,
		args_info.hostname_arg);
	if (do_connect(sockfd, &their_addr) < 0)
		exit(EXIT_FAILURE);
	if (send_buffer(sockfd, "GET /////////////////7\n") < 0)
		exit(EXIT_FAILURE);
	/* Receive ogg header */
	if ((numbytes = recv_buffer(sockfd, header, STRINGSIZE - 1)) < 0)
		exit(EXIT_FAILURE);
	write(STDOUT_FILENO, header, numbytes);
	/* write server output to stdout */
	while ((numbytes = recv_buffer(sockfd, buf, STRINGSIZE)) > 0) {
		write(STDOUT_FILENO, buf, numbytes);
		if (send_buffer(sockfd, header) < 0)
			exit(EXIT_FAILURE);
	}
	if (numbytes < 0) {
		clog(ERROR, "%s: Read %i (%s)\n", __func__, numbytes, strerror(errno));
		exit(EXIT_FAILURE);
	}
	if (!numbytes)
		clog(NOTICE, "%s: Connection closed by peer\n", __func__);
	close(sockfd);
	return 0;
}
